#!/usr/bin/perl
# Modification Date: 01/14/15

use warnings;
use strict;

#Constants
my $product_id = "08";
my $product_name = "IBM Spectrum Protect for Space Management";
my $opt_prefix = "--";
my %defined_options = (
      "namespace"          => [255, "true", "<TSM node name>", "required", "IBM Spectrum Protect node name."],
      "applicationentity"  => [255, "true", "<filespace>",     "required", "The mount point of file system."],
      "directory"          => [255, "true", "<directory>",     "required", "Directory where the output should be written to."]
);

# Global arrays 
my @dsmdf_result;
my @list_of_gpfs_nodes;

#Valriable
my $item_type = "backup";
my $selectMode = "";
my %user_options = ();


print"\n";
print"********************************************************************\n";
print"********      IBM Spectrum Protect Suite - Front End       *********\n";
print"********          Terabyte (TB) Capacity Report            *********\n";
print"********************************************************************\n\n";
print"$product_name\n\n";

parse_cmd_line();
get_list_of_hsm_nodes();
query_usage_data();

#-----------------------------------------------------------------------+
# parse_cmd_line                                                        |
# ===================================================================== |
# Purpose: Parse command line parameter.                                |
#                                                                       |
#-----------------------------------------------------------------------+
sub parse_cmd_line 
{
   my $num_args = @ARGV;

   if ($num_args != 3)
   {
      print_help();
      exit(12);
   }

   foreach (@ARGV)
   {
      my $option_str = $_;
      if ($option_str =~ /^$opt_prefix(([a-z]|[A-Z])+)=(.+)$/)
      {
         my $option_name = $1;
         my $option_value = $3;
         if(validate_option_with_value($option_name, $option_value))
         {
            $user_options{$option_name} = $option_value;
         }
         else
         {
            print "Invalid option/value pair: $_\n";
            print_help();
            exit(12);
         }
      }
      else
      {
         print "Invalid option/value pair: $_\n";
         print_help();
         exit(12);
      }
   }

   $user_options{'namespace'} = uc($user_options{'namespace'});
   validate_node_name($user_options{'namespace'});

   print"Input values:\n";
   my @options = keys %user_options;

   foreach (@options)
   {
      print"${opt_prefix}$_=$user_options{$_}\n";
   }

   if ($user_options{'applicationentity'} eq "*")
   {
      $selectMode = "all"
   }
   else
   {
      $selectMode = "sel"
   }
}

#-----------------------------------------------------------------------+
# get_list_of_hsm_nodes                                                 |
#=======================================================================|
# Purpose: return list of hsm nodes                                     |
#                                                                       |
#-----------------------------------------------------------------------+
sub get_list_of_hsm_nodes
{
   my @tab;
   my $node_name;
   my @dsmmigfs_result;

   # This script does not check TSM node name for each 
   # GPFS node. We believe that all GPFS nodes have the 
   # same TSM node name.

   if ( $selectMode eq "all" )
   {
      #run the command "dsmmigfs q -f"
      @dsmmigfs_result = `dsmmigfs q -f`;
      if($? != 0)
      {
         print "The HSM command \"dsmmigfs q -f\" was finished with error!\n";
         exit(12);
      }

      #get all nodes of cluster
      foreach (@dsmmigfs_result)
      {
         if ($_ =~ /Node:/)
         {
            @tab=split(/\s+/,$_);
            $node_name = $tab[1];
            push @list_of_gpfs_nodes, $node_name;
         }
      }
   }
   else
   {
      #run the command "dsmmigfs q -d /FSName" for specified file system
      my $cmd = "dsmmigfs q -d ";
      $cmd .= "$user_options{'applicationentity'}";
      @dsmmigfs_result = `$cmd`;
      if($? != 0)
      {
         print "The HSM command \"dsmmigfs q -d\" was finished with error!\n";
         exit(12);
      }

      #get node name which is owner of specified FS
      foreach (@dsmmigfs_result)
      {
         if ($_ =~ /Owner Node:/)
         {
            @tab=split(/\s+/,$_);
            $node_name = $tab[2];
            push @list_of_gpfs_nodes, $node_name;
         }
      }
   }
}

#-----------------------------------------------------------------------+
# query_usage_data                                                      |
# ===================================================================== |
# Purpose: Collect the usage data information for single gpfs node.     |
#                                                                       |
#-----------------------------------------------------------------------+
sub query_usage_data 
{
   print"\nQuery usage data\n\n";

   #run "dsmdf -detail" 
   foreach (@list_of_gpfs_nodes)
   {
      my $node_name = $_;
      my $cmd_str = "ssh " . $node_name . "  dsmdf -detail ";
      if ( $selectMode eq "sel")
      {
         $cmd_str .= "$user_options{'applicationentity'} ";
      }

      @dsmdf_result = `$cmd_str`;
      parse_usage_data();
   }

   if($? != 0)
   {
      print "Error of collecting data!\n";
      exit(12);
   }
   
}

#-----------------------------------------------------------------------+
# parse_usage_data                                                      |
# ===================================================================== |
# Purpose: Parse the collected usage data and save them in the xml file |
#                                                                       |
#-----------------------------------------------------------------------+
sub parse_usage_data
{
   my $applicationentity = "";
   my $size = 0;
   my $numberofobjects = 0;

   #parce output of the command "dsmdf -detail"
   foreach (@dsmdf_result)
   {
      my @line = split(/:/, $_);

      if ($_ =~ /HSM Filesystem/)
      {
         $line[1] =~ s/\s//g;
         $applicationentity = $line[1];
      }
      elsif ($_ =~ /Migrated Files/)
      {
         $numberofobjects += $line[1];
      }
      elsif ($_ =~ /Premigrated Files/)
      {
         $numberofobjects += $line[1];
      }
      elsif ($_ =~ /Migrated Size/)
      {
         $size += $line[1];
      }
      elsif ($_ =~ /Premigrated Size/)
      {
         $size += $line[1];
      }
   }

   #save data of last file system in XML file
   if ($applicationentity ne "")  
   {
      if ( $selectMode eq "all" )
      {
         $applicationentity = "*";
      }
      $size = sprintf("%.f", $size/1024);    #translate size into MB
      create_xml_file($applicationentity, $size, $numberofobjects);
   }
}

#-----------------------------------------------------------------------+
# create_xml_file                                                       |
# ===================================================================== |
# Purpose: Parse the collected usage data and save them in the xml file |
#                                                                       |
# Parameters:                                                           |
#         applicationentity -> fs name                                  |
#         size              -> migrated and premigrated size            |
#         numberofobjects   -> common number of migrated and premigrated|
#                              objects                                  |
#-----------------------------------------------------------------------+
sub create_xml_file
{
   my($applicationentity, $size, $numberofobjects) = @_;

   my $command_str = "./dsmfecc --create ";
   $command_str .= "--namespace=$user_options{'namespace'} ";
   $command_str .= "--productid=$product_id ";
   $command_str .= "--directory=$user_options{'directory'} ";
   $command_str .= "--applicationentity=$applicationentity ";
   $command_str .= "--numberofobjects=$numberofobjects ";
   $command_str .= "--size=$size ";
   $command_str .= "--type=hsm ";

   my @dsmfecc_result = `$command_str`;

   if ($? != 0)
   {
      print "@dsmfecc_result\n";
   }
   else
   {
      print "Query result:\n";
      print "   Number of objects: $numberofobjects\n";
      print "   Size (MB): $size\n";
      print "@dsmfecc_result\n";
   }
}

#-----------------------------------------------------------------------+
# print_help                                                            |
# ===================================================================== |
# Purpose: Display command line help                                    |
#                                                                       |
#-----------------------------------------------------------------------+
sub print_help 
{
   print"Usage:\n";
   print "dsmfecc-${product_id}.pl\n";
   my @options = keys %defined_options;

   foreach (@options)
   {
      my $option = " ${opt_prefix}$_";
      if ($defined_options{$_}[1] eq "true")
      {
         $option .= "=$defined_options{$_}[2]";
      }
      printf("%-40s", $option);

      my $desc = "$defined_options{$_}[4]";
      $desc .= "\n";
      print "$desc";
   }
}

#-----------------------------------------------------------------------+
# validate_option_with_value                                            |
# ===================================================================== |
# Purpose: Validate options which are inputed by user                   |
#                                                                       |
#-----------------------------------------------------------------------+

sub validate_option_with_value
{
   my $rc = "";

   if (is_valid_option($_[0]))
   {
      if ($defined_options{$_[0]}[1] eq "true")
      {
         $rc = 1;
      }
   }
   return $rc;
}

sub is_valid_option
{
   my $option = $_[0];
   my $rc = "";

   if (exists($defined_options{$option}))
   {
      $rc = 1;
   }
   return $rc;
}

#-----------------------------------------------------------------------+
# validate_node_name                                                    |
# ===================================================================== |
# Purpose: valiidate that user input correct TSM node name              |
#                                                                       |
# Parameters:                                                           |
#          nodename -> node name which was inputed by user              |
#                                                                       |
#-----------------------------------------------------------------------+
sub validate_node_name
{
   my($nodename) = @_;

   my @dsmc_q_sess_result = `dsmc q sess`;
   foreach (@dsmc_q_sess_result)
   {
      if ($_ =~ /Node Name/)
      {
         my @line = split(/:/, $_);
         my $nodename1 = $line[1];
         $nodename1 =~ s/\s//g;
         if ($nodename ne $nodename1)
         {
            print "Namespace $nodename is not used on this HSM node. $nodename1 is defined in the dsm.sys\n";
            exit(12);
         }
      }

   }
}
