#!/usr/bin/perl

use warnings;
use strict;
use POSIX;

my $product_id = "16";
my $item_type = "fcm";
my $product_name = "IBM Spectrum Protect Snapshot for Oracle";

my @result_fe = "";
my @result_lun = "";
my @result_be = "";

my $opt_prefix = "--";
my %defined_options = (
     "applicationusername" => [255, "true", "<database user>", "required", "User Name for logging on to the database."],
     "applicationpassword" => [255, "true", "<database user password>", "optional", "Password for logging on to the database. (optional)"], 
     "namespace" => [255, "true", "<unique namespace definition>", "required", "Unique name to ensure data is counted only once and result can be identified."],     "directory" => [255, "true", "<directory>", "required", "Directory where the output should be written to."],
     "fcminstance" => [255, "true", "<fcminstance>", "required", "Directory where the IBM Spectrum Protect Snapshot instance is located."],
     "befactor" => [255, "true", "<befactor 0-1>", "optional", "Enter the BE Factor."]
);

my %user_options = ();

my %capacity_counting = (
     "item_type"     => "",
     "item_name"     => "",
     "item_countFE"  => "",
     "item_sizeFE"   => "",
     "item_countLUN" => "",
     "item_sizeLUN"  => "",
     "item_countBE"  => "",
     "item_sizeBE"   => "",
     "offload"       => ""
);

print "\n";
print "********************************************************************\n";
print "********      IBM Spectrum Protect Suite - Front End       *********\n";
print "********          Terabyte (TB) Capacity Report            *********\n";
print "********************************************************************\n\n";
print "$product_name\n\n";

parse_cmd_line();
query_fe();
parse_fe();
query_lun();
parse_lun();
query_be();
parse_be();
create_xml_file();

sub parse_cmd_line
{
   my $num_args = @ARGV;
   
   if ($num_args == 0)
   {
      print_help();
      exit(12);
   }
   
   foreach (@ARGV)
   {
      my $option_str = $_;
      
      if ($option_str =~ /^$opt_prefix(([a-z]|[A-Z])+)$/)
      {
         my $option_name = $1;
         if (validate_option($option_name))
         {
            $user_options{$option_name} = "";
         }
         else
         {
            print "Invalid option/value pair 1: $_\n";
            print_help();
            exit(12);
         }
      }
      elsif ($option_str =~ /^$opt_prefix(([a-z]|[A-Z])+)=(.+)$/)
      {
         my $option_name = $1;
         my $option_value = $3;

         if(validate_option_with_value($option_name, $option_value))
         {
            $user_options{$option_name} = $option_value;
         }
         else
         {
            print "Invalid option/value pair 2: $_\n";
            print_help();
            exit(12);
         }
      }
      else
      {
         print "Invalid option/value pair 3: $_\n";
         print_help();
         exit(12);
      }
   }

   my @options = keys %defined_options;
   foreach (@options)
   {
      if ($defined_options{$_}[3] eq "required")
      {
         if (!exists($user_options{$_}))
         {
            print "Missing option: $opt_prefix$_\n";
            print_help();
            exit(12);
         }
      }
   }
}

sub query_fe
{
   print "\nQuery Usage data... \n\n";
   my $sqlStr = "select sum(bytes)/1024/1024 " . '"Meg"' . " from dba_data_files;\nEXIT";
   open(SQLCMD, ">cmd.sql") or die "Couldn't open file file.txt, $!";
   print SQLCMD $sqlStr;
   close SQLCMD;

   my $cmd_str ="";

   if (!exists($user_options{'applicationpassword'}))
   {
      $cmd_str .= "sqlplus / as $user_options{'applicationusername'} \@cmd.sql";
   }
   else
   {
      $cmd_str .= "sqlplus / as $user_options{'applicationusername'}/$user_options{'applicationpassword'} \@cmd.sql";
   }

   @result_fe = `$cmd_str`;

   if($? != 0)
   {
      print "@result_fe\n";
      exit(12);
   }
}

sub query_lun
{
   @result_lun = `$user_options{'fcminstance'}/fmquery -c`;

   if ($? != 0)
   {
      print "@result_lun\n";
      exit(12);
   }
}

sub query_be
{
   @result_be = `$user_options{'fcminstance'}/fcmcli -f inquire_detail`;
   
   if ($? != 0)
   {
      print "@result_be\n";
      exit(12);
   }
}
 
sub parse_fe
{
   my $split_lines =0;
   for (@result_fe)
   { 
      if ($_ =~ /-----/)
      {
         $split_lines = 1;
         next;
      }
      if ($split_lines)
      {
         my $size_fe = $_;
         chomp($size_fe);
         $size_fe =~ s/^\s+//;
         $capacity_counting{'item_countFE'} = 1;
         $capacity_counting{'item_sizeFE'} = $size_fe;
         $split_lines = 0;
      }
   }
   
   $capacity_counting{'item_type'} = $item_type;
   $capacity_counting{'item_name'} = 0;
   print "Query Result:\n\n";
   print "FE:\n";
   print "Number of Objects:             $capacity_counting{'item_countFE'}\n";
   print "Size (MB):                     $capacity_counting{'item_sizeFE'}\n\n";
}

sub parse_lun
{
   for (@result_lun)
   {
      if ($_ =~ /fcm_lun_capacity,/)
      {
         my $size_lun = $_;
         $size_lun = $'/1024/1024;
         chomp($size_lun);
         $capacity_counting{'item_sizeLUN'} = $size_lun;
      }
      if ($_=~ /fcm_lun_objects,/)
      {
         my $object_lun = $_;
         $object_lun = $';
         chomp($object_lun);
         $capacity_counting{'item_countLUN'} = $object_lun;
      }
      if ($_ =~ /tsm,/)
      {
         my $offload = $_;
         chomp($offload);
         $offload = $';
         if ($offload =~ /yes/)
         {
            $capacity_counting{'offload'} = 'y';
         }
         else
         {
            $capacity_counting{'offload'} = 'n';
         }
      }
   }
   
   
   print "LUN:\n";
   print "Number of Objects:             $capacity_counting{'item_countLUN'}\n";
   print "Size (MB):                     $capacity_counting{'item_sizeLUN'}\n\n";
}

sub parse_be
{
   my $object_be = 0;
   my $size_be = 0;
   my $befactor = $user_options{'befactor'};
   
   $befactor = 1 if not defined $befactor;
     
   if ($befactor <= 0 || $befactor >= 1)
   {
      $befactor = 1;   
   }    

   for (@result_be)
   {
      if ($_ =~/#BACKUP/)
      {
         if ($_ !~ /#BACKUP -/)
         {    
           $object_be++;
         }
      }
   }
   
   $size_be = $object_be * $capacity_counting{'item_sizeFE'} * $befactor;

   $capacity_counting{'item_countBE'} = $object_be;
   $capacity_counting{'item_sizeBE'} = int $size_be;
  
   print "BE:\n";
   print "Number of Objects:             $capacity_counting{'item_countBE'}\n";
   print "Size (MB):                     $capacity_counting{'item_sizeBE'}\n";
   print "Backup to IBM Spectrum Protect Server detected: $capacity_counting{'offload'}\n"; 
}

sub create_xml_file
{
   print"\nCreate XML file... \n\n";
   my $command_str = "./dsmfecc --create ";
   $command_str .= "--namespace=$user_options{'namespace'} ";
   $command_str .= "--productid=$product_id ";
   $command_str .= "--directory=$user_options{'directory'} ";
   $command_str .= "--applicationentity=$capacity_counting{'item_name'} ";
   $command_str .= "--numberofobjects=$capacity_counting{'item_countFE'} ";
   $command_str .= "--size=$capacity_counting{'item_sizeFE'} ";
   $command_str .= "--type=$capacity_counting{'item_type'} ";
   $command_str .= "--fcmbenumberofobjects=\"$capacity_counting{'item_countBE'}\" ";
   $command_str .= "--fcmbesize=\"$capacity_counting{'item_sizeBE'}\" ";
   $command_str .= "--fcmlunnumberofobjects=\"$capacity_counting{'item_countLUN'}\" ";
   $command_str .= "--fcmlunsize=\"$capacity_counting{'item_sizeLUN'}\" ";
   $command_str .= "--offload=\"$capacity_counting{'offload'}\"";   
   
   system($command_str);

   if ($? == 0)
   {
       print "XML file created in $user_options{'directory'}\n";
   }        
}

sub print_help
{
   print "Usage:\n";
   print "dsmfecc-${product_id}.pl\n";
   my @options= keys %defined_options;
   foreach (@options)
   {
      my $option = "";
      if($defined_options{$_}[1] eq "true")
      {
         if($defined_options{$_}[3] eq "optional")
         {
            $option .= "${opt_prefix}$_";
            $option .= "=$defined_options{$_}[2]";
         }
         else
         {
            $option .= "${opt_prefix}$_";
            $option .= "=$defined_options{$_}[2]";
         }
      }
   printf("%-50s", $option);    
   my $desc = "$defined_options{$_}[4]";
   $desc .= "\n";
   print "$desc";
   }
}

sub validate_option
{
   my $rc = "";
   my $option_name= $_[0];
   if (is_valid_option($option_name))
   {
      if ($defined_options{$option_name}[1] ne "true")
      {
         $rc = 1;
      }
   }
   return $rc;
}

sub validate_option_with_value
{
   my $rc = "";
   if (is_valid_option($_[0]))
   {
      if ($defined_options{$_[0]}[1] eq "true")
      {
         $rc = 1;
      }
   }
   return $rc;
}

sub is_valid_option
{
   my $option = $_[0];
   my $rc = "";
   if (exists($defined_options{$option}))
   {
      $rc = 1;
   }
   return $rc;
}
